%%%%%%%%%%%%%%%% Source code to reproduce Figure 8 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written in Matlab R2020a

clearvars;close all;clc % cleaning

% define figure parameters
figp.twocolumn = 15.5; % size for for two-column figure
figp.fontsize = 7; % fontsize for general use
figp.fontsize_axis = 8; % fontsize for axis label
figp.fontname = 'Arial'; % Use Arial font

% set symbol and color for each object category
symbol.fruits = 'o';symbol.barks = 'o';
symbol.leaves = 'o';symbol.flowers = 'o';symbol.plantfruits = 'o';
symbol.vegetables = 'o';symbol.stones = 'd';
symbol.butterflies = '^';symbol.birds = '^';
symbol.beetles = '^';

cmap.fruits = [176 197 98]/255;cmap.barks = [159 0 55]/255;
cmap.leaves = [102 184 114]/255;cmap.flowers = [217 176 76]/255;cmap.plantfruits = [0 163 133]/255;
cmap.vegetables = [83 58 149]/255;cmap.stones = [0 108 164]/255;
cmap.butterflies = [228 135 41]/255;cmap.birds = [223 65 34]/255;
cmap.beetles = [214 0 59]/255;

cmap_country.Argentina = [0.2627,0.5765,0.7647];
cmap_country.Japan = [0.8392,0.3765,0.3020];
cmap_country_edge.Argentina = [0.8392,0.3765,0.3020];
cmap_country_edge.Japan = [0.2627,0.5765,0.7647];

% load reflectances measured in Japan
% Morimoto et al., “Spectral measurement of daylights and surface properties of natural objects in Japan”, Optics Express, 30, 3183-3204 (2022). https://doi.org/10.1364/OE.441063
load('ref_Japan');

% load pre computed xy chromaticities for daylights (4500K to 20000K in 500 nm step)
load('CIEdaylight_xyY_4500to20000K')

% define category for each dataset
category.Argentina = {'barks','flowers','fruits','leaves','plantfruits','vegetables','stones','beetles','birds','butterflies'};
category.Japan = {'barks','flowers','fruits','leaves','vegetables','stones'};

% load the Argentina dataset from excel file
filename_prefix = 'Database_Objetos_Naturales_Final_';
species = {'Animals', 'Plants', 'Stones'};
mapping = containers.Map( ...
    {'Barks (Ba)','Flowers (Fl)','Fruits (F)','Leaves (L)','Plant fruits (Fr)','Vegetables (V)','Stones (S)','Beetles (Be)','Birds (B)','Butterflies (Bu)'}, ...
    category.Argentina);
wls = (400:4:1000)';

Data_Argentina = Get_ArgentinaDataset(filename_prefix,species,mapping,wls);

% put each dataset to Data
Data.Japan = Data_Japan;
Data.Argentina = Data_Argentina;

% compute xy chromaticity and concatenate over categories to "all_xyY"
C = [];all_xyY = [];
for country = {'Japan','Argentina'}
    data = [];
    for c = category.(country{1})
        C.(country{1}).(c{1}) = SpectralVectortoXYZandxyY_400to1000nm(Data.(country{1}).(c{1}),4);
        data = [data;C.(country{1}).(c{1}).xyY];
    end
    all_xyY.(country{1}) = data;
end

%% Plot chromatic distribution for each dataset and save the figures (Fig 10)
for country = {'Argentina','Japan'}
    fig = figure;
    
    % plot xy chromaticity diagram (taken from PsychToolbox3)
    plotxyChromaticity;hold on;

    ax = gca;
    
    % compute gamut informaiton
    k_gamut = convhull(all_xyY.(country{1})(:,1), all_xyY.(country{1})(:,2));
    C_gamut = polyshape(all_xyY.(country{1})(k_gamut,1), all_xyY.(country{1})(k_gamut,2));

    cnt = 0;
    for c = category.(country{1})
        cnt = cnt + 1;

        dc = cmap.(c{1});
        x = C.(country{1}).(c{1}).xyY(:,1);
        y = C.(country{1}).(c{1}).xyY(:,2);

        % draw gamut for the first category
        if cnt == 1
            plot(C_gamut, 'FaceColor', 'none', 'EdgeColor', cmap_country.(country{1}),'LineWidth',0.5,'LineStyle','-','HandleVisibility','off');
        end

        % plot data
        scatter(x,y,20,dc,symbol.(c{1}),'filled','MarkerEdgeColor',[.3 .3 .3],'MarkerFaceAlpha',0.7,'HandleVisibility','off','LineWidth',0.1);hold on;

        % plot daylight locus
        plot(CIEdaylight_xyY_4500to20000K(:,1),CIEdaylight_xyY_4500to20000K(:,2),'Color','m','LineWidth',1,'HandleVisibility','off');

        % plot equal energy white
        scatter(1/3,1/3,40,'k+','LineWidth',1,'HandleVisibility','off');hold on;

        % set figure parameters
        ax.XLim = [0 0.9];ax.YLim = [0 0.9];
        ax.XTick = [0 0.45 0.9];ax.YTick = [0 0.45 0.9];

        ax.XTickLabel = char('0.00','0.45','0.90');
        ax.YTickLabel = char('0.00','0.45','0.90');

        xlabel('CIE1931 x','FontSize',figp.fontsize_axis);
        ylabel('CIE1931 y','FontSize',figp.fontsize_axis);

        fig.PaperType       = 'a4';fig.PaperUnits = 'centimeters';
        fig.Units           = 'centimeters';fig.Color  = 'w';
        fig.InvertHardcopy  = 'off';

        fig.Position = [10,10,figp.twocolumn/2,figp.twocolumn/2];
        fig.PaperPosition   = [0,10,8.45,8.45];

        ax.FontName = figp.fontname;ax.FontSize = figp.fontsize;
        ax.Color = ones(3,1);

        ax.Units = 'centimeters';
        axis square;
        ax.Position = [1 1 6.3 6.3];

        ax.TickDir = 'out';
        box off;grid off
        ax.LineWidth = 0.3;

        ax.YAxis.Visible = 'on';

        grid off
    end
    
    % Save figure
    filename = [country{1},'_chromaticity'];
    exportgraphics(fig,fullfile('fig',[filename,'.pdf']),'ContentType','vector')
end

%% Plot overlapping gamut area and save the figures (Fig 10, inserted figures)
% Compute the convex hull for each dataset
for country = {'Argentina','Japan'}
    x_all.(country{1}) = all_xyY.(country{1})(:,1);
    y_all.(country{1}) = all_xyY.(country{1})(:,2);
end

for country = {'Argentina','Japan'}
    convhull_k.(country{1}) = convhull(x_all.(country{1}), y_all.(country{1}));
    gamut.(country{1}) = polyshape(x_all.(country{1})(convhull_k.(country{1})), y_all.(country{1})(convhull_k.(country{1})));
    gamut_area.(country{1}) = area(gamut.(country{1}));
end

% Compute the area of the overlapping region
overlapGamut = intersect(gamut.Argentina, gamut.Japan);
overlapArea = area(overlapGamut);

% Plot overlapping region for each dataset
for country = {'Argentina','Japan'}
    fig = figure;
        
    % Plot xy chromaticity diagram
    plotxyChromaticity;hold on
    
    % Plot gamut
    plot(gamut.(country{1}), 'FaceColor', cmap_country.(country{1}), 'FaceAlpha', 0.1, 'EdgeColor', cmap_country.(country{1}),'LineWidth',0.8);

    % Plot the overlapping region with a different color
    plot(overlapGamut, 'FaceColor', cmap_country_edge.(country{1}), 'FaceAlpha', 0.5, 'EdgeColor', 'none');

    % Plot equal energy white
    scatter(1/3,1/3,50,'k+','LineWidth',1.5,'HandleVisibility','off');hold on;

    ax = gca;

    % set figure parameters
    ax.XLim = [0 0.9];ax.YLim = [0 0.9];
    ax.XTick = [0 0.45 0.9];ax.YTick = [0 0.45 0.9];

    ax.XTickLabel = char('0.00','0.45','0.90');
    ax.YTickLabel = char('0.00','0.45','0.90');

    xlabel('','FontSize',figp.fontsize_axis);
    ylabel('','FontSize',figp.fontsize_axis);

    fig.PaperType       = 'a4';fig.PaperUnits = 'centimeters';
    fig.Units           = 'centimeters';fig.Color  = 'w';
    fig.InvertHardcopy  = 'off';
    fig.Position = [10,10,figp.twocolumn/4,figp.twocolumn/4];
    fig.PaperPosition   = [0,10,8.45,8.45];

    ax.FontName = figp.fontname;ax.FontSize = figp.fontsize;
    ax.Color = ones(3,1);

    ax.Units = 'centimeters';
    axis square;
    ax.Position = [0.4 0.1 3.9 3.9];

    ax.TickDir = 'out';
    box off;grid off
    ax.LineWidth = 0.3;

    ax.YAxis.Visible = 'off';
    ax.XAxis.Visible = 'off';

    grid off
    
    % Save figure
    filename = ['gamut_',country{1},'_chromaticity'];
    exportgraphics(fig,fullfile('fig',[filename,'.pdf']),'ContentType','vector')
end

% Display messages
fprintf('Area of gamut for Argentina dataset: %f\n', gamut_area.Argentina);
fprintf('Area of gamut for Japanese dataset: %f\n', gamut_area.Japan);
fprintf('Percentage of overlapping region with Japanese gamut: %f\n', overlapArea/gamut_area.Argentina*100);
fprintf('Percentage of overlapping region with Argentina gamut: %f\n', overlapArea/gamut_area.Japan*100);
